﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Threading;

namespace _30PhotoStudio
{
    public partial class Form1 : Form
    {
        // Privátní proměnné (výchozí cesta, kolekce panelů obrázků atd.)
        string cesta = @"C:\Data k testovani\T1";
        string[] soubory;
        List<Panel> panely = new List<Panel>();
        int selectedPanel = -1;
        
        /// <summary>
        /// První spuštění formuláře - náhledy snímků, pokud se spouští z příkazové řádky, předáváme mu parametr
        /// </summary>
        public Form1()
        {
            InitializeComponent();
        }
        public Form1(string cesta)
        {
            // Parametr z příkazové řádky
            this.cesta = cesta;
            InitializeComponent();

            soubory = Directory.GetFiles(cesta, "*.jp*g", SearchOption.TopDirectoryOnly);
            this.Text = cesta;
            this.Cursor = Cursors.WaitCursor;

            NacistObrazky();
        }

        private void label2_Click(object sender, EventArgs e)
        {
            // Načte dialog stromu složek
            folderBrowserDialog1.SelectedPath = cesta; //Application.StartupPath;
            folderBrowserDialog1.ShowDialog();

            soubory = Directory.GetFiles(folderBrowserDialog1.SelectedPath, "*.jp*g", SearchOption.TopDirectoryOnly);
            this.Text = folderBrowserDialog1.SelectedPath;

            this.Cursor = Cursors.WaitCursor;
            NacistObrazky();
        }

        /// <summary>
        /// Podle zvolené cesty (vybrané v dialogu nebo z příkazové řádky) načte náhledy obrázků
        /// </summary>
        private void NacistObrazky()
        {
            // Vyčistíme starý seznam obrázků
            foreach (Panel p in panely)
                this.Controls.Remove(p);
            panely.Clear();

            // Procházíme soubory a generujeme dynamicky seznam obrázků (panely)
            int index = 0;
            foreach (string s in soubory)
            {
                Panel p = new Panel();
                p.Width = panel1.ClientSize.Width;
                p.Height = 120;
                if ((index % 2) == 0)
                    p.BackColor = Color.WhiteSmoke;
                else
                    p.BackColor = Color.LightYellow;

                PictureBox pic = new PictureBox();
                try
                {
                    pic.BackgroundImage = Image.FromFile(s);
                }
                catch
                {
                    continue;
                }
                pic.BackgroundImageLayout = ImageLayout.Stretch;
                pic.BorderStyle = BorderStyle.FixedSingle;
                pic.Left = 10;
                pic.Top = 10;
                pic.Height = 100;
                pic.Width = (int)(((double)pic.BackgroundImage.Width / (double)pic.BackgroundImage.Height) * 100d);
                p.Controls.Add(pic);

                // Popisky obrázku (název, cesta...)
                #region Popisky
                Label cesta = new Label();
                cesta.Left = pic.Width + 20;
                cesta.Top = 20;
                cesta.Text = s;
                cesta.AutoSize = true;
                p.Controls.Add(cesta);

                Label c2 = new Label();
                c2.Left = pic.Width + 20;
                c2.Top = 40;
                try
                {
                    c2.Text = s.Substring(s.LastIndexOf('\\') + 1);
                }
                catch { }
                c2.AutoSize = true;
                p.Controls.Add(c2);

                Label c3 = new Label();
                c3.Left = pic.Width + 20;
                c3.Top = 60;
                c3.Text = File.GetCreationTime(s).ToString();
                c3.AutoSize = true;
                p.Controls.Add(c3);

                Label c4 = new Label();
                c4.Left = pic.Width + 20;
                c4.Top = 80;
                c4.Text = pic.BackgroundImage.Width.ToString() + " x " + pic.BackgroundImage.Height.ToString();
                c4.AutoSize = true;
                p.Controls.Add(c4);

                Label c5 = new Label();
                c5.Left = pic.Width + 20;
                c5.Top = 100;
                FileInfo fi = new FileInfo(s);
                c5.Text = fi.Length.ToString() + " byte";
                c5.AutoSize = true;
                p.Controls.Add(c5);
                #endregion

                // Určení pozice panelu s obrázkem, přidání do kolekce
                p.Left = 0;
                if (panely.Count > 0)
                    p.Top = panely[panely.Count - 1].Top + 120;
                else
                    p.Top = 0;
                p.Tag = (string)s;
                p.TabIndex = index;

                p.DoubleClick += new EventHandler(p_DoubleClick);
                p.Click += new EventHandler(p_Click);

                index++;
                panel1.Controls.Add(p);
                panely.Add(p);

                // Možnost pracovat i v rámci načítání, aplikace reaguje
                Application.DoEvents();
            }

            // Zvolíme první panel jako vybraný
            if (panely.Count > 0)
            {
                selectedPanel = 0;
                panely[selectedPanel].BackColor = Color.LightBlue;
            }

            // Upravíme šířku prvních několika panelů, aby se přizpůsobila vykreslovací oblasti
            foreach (Panel p in panely)
            {
                if (p.Width > panel1.ClientSize.Width)
                    p.Width = panel1.ClientSize.Width;
                else
                    break;
            }
            this.Cursor = Cursors.Default;
        }

        void p_Click(object sender, EventArgs e)
        {
            // Kliknutí na náhled
            SelectPanel(selectedPanel, ((Panel)sender).TabIndex);
        }

        void p_DoubleClick(object sender, EventArgs e)
        {
            // Dvojklik na náhled
            SelectPanel(selectedPanel, ((Panel)sender).TabIndex);
            Form2 f = new Form2((string)panely[selectedPanel].Tag, panely[selectedPanel].TabIndex, soubory, this);
            f.ShowDialog();
        }

        private void Form1_KeyPress(object sender, KeyPressEventArgs e)
        {
            MyKeyPress(e);
        }

        /// <summary>
        /// Odchyt kláves pro ovládání programu
        /// </summary>
        private void MyKeyPress(KeyPressEventArgs e)
        {
            if ((e.KeyChar == '8') && (selectedPanel != 0))
            {
                SelectPanel(selectedPanel, selectedPanel - 1);
            }
            else if ((e.KeyChar == '2') && (selectedPanel != panely.Count - 1))
            {
                SelectPanel(selectedPanel, selectedPanel + 1);
            }
            else if (e.KeyChar == '\r') //enter
            {
                Form2 f = new Form2((string)panely[selectedPanel].Tag, panely[selectedPanel].TabIndex, soubory, this);
                f.ShowDialog();
            }
            else if (e.KeyChar == 27) //esc
            {
                this.Close();
            }
        }

        /// <summary>
        /// Výběr obrázků (myší nebo klávesnicí)
        /// </summary>
        private void SelectPanel(int s, int p)
        {
            if ((s % 2) == 0)
                panely[s].BackColor = Color.WhiteSmoke;
            else
                panely[s].BackColor = Color.LightYellow;
            selectedPanel = p;
            panely[selectedPanel].BackColor = Color.LightBlue;
        }

        private void label3_Click(object sender, EventArgs e)
        {
            //Nápověda k programu
            MessageBox.Show("Vítejte v prohlížeči fotek 30PhotoStudio.\n\nV tomto okně vidíte zobrazení složky obrázků, jinou složku můžete zvolit kliknutím na -Načíst obrázky-. Program můžete spustit i z příkazové řádky s parametrem složky nebo obrázku. Mezi obrázky můžete přecházíte klávesami 8 a 2 (jako šipky), dvojklikem nebo stisknutím Enter se dostanete do prohlížeče fotek. Tam můžete obrázek přibližovat, zobrazit další, předchozí atd. Klávesou Esc tento program zavřete.");
        }
    }
}
