﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Drawing.Drawing2D;

namespace _30PhotoStudio
{
    public partial class Form2 : Form
    {
        // Privátní proměnné
        double zoom = 1d;
        string cesta = "";
        int kolikaty = 0, kolikCelkem = 0;
        string[] soubory;
        bool zobrazitVse = true;
        Form parentForm;
        bool preklopitHor = false, preklopitVer = false;
        int otocit = 0;
        bool nacteno = false, poprvePosledni = true;
        bool grayscale = false;

        Bitmap tempNextBitmap, tempPrevBitmap, tempFirstBitmap, tempLastBitmap;
        int tempPrevIndex = -1, tempNextIndex = -1;

        /// <summary>
        /// Přetížený konstruktor, předává se mu seznam složek a adresa obrázku k zobrazení
        /// </summary>
        public Form2()
        {
            InitializeComponent();
        }
        public Form2(string cesta, int kolikaty, string[] soubory, Form parentForm)
        {
            this.cesta = cesta;
            this.kolikaty = kolikaty;
            this.kolikCelkem = soubory.Length;
            this.soubory = soubory;
            this.parentForm = parentForm;
            InitializeComponent();
        }
        
        private void Form2_Load(object sender, EventArgs e)
        {
            //Načtení Formu, vykreslí obrázek
            if (cesta != "")
                this.Text = cesta + " ... " + (kolikaty + 1) + ". v adresáři, " + kolikCelkem + " celkem.";
            Vykreslit();

            nacteno = true;
        }

        /// <summary>
        /// Metoda zajišťující správné vykreslení obrázku, zoom atd.
        /// </summary>
        private void Vykreslit()
        {
            Bitmap b = new Bitmap(1, 1);
            try
            {
                // Cachování bitmap, které už známe, pro rychlejší načítání
                if ((tempNextIndex == kolikaty) && (tempNextBitmap != null))
                    b = tempNextBitmap;
                else if ((tempPrevIndex == kolikaty) && (tempPrevBitmap != null))
                    b = tempPrevBitmap;
                else if ((kolikaty == 0) && (tempFirstBitmap != null))
                    b = tempFirstBitmap;
                else if ((kolikaty == soubory.Length - 1) && (tempLastBitmap != null))
                    b = tempLastBitmap;
                else
                    b = new Bitmap(Image.FromFile(cesta));
            }
            catch
            {
                return;
            }

            // Řešení překlopení, otočení obrázku
            if (preklopitHor == true)
            {
                Bitmap b2 = new Bitmap(b.Width, b.Height);
                Graphics g = Graphics.FromImage(b2);
                g.DrawImage(b, b.Width - 1, 0, b.Width * -1, b.Height);
                b = b2;
            }
            if (preklopitVer == true)
            {
                Bitmap b2 = new Bitmap(b.Width, b.Height);
                Graphics g = Graphics.FromImage(b2);
                g.DrawImage(b, 0, b.Height - 1, b.Width, b.Height * -1);
                b = b2;
            }
            if (otocit != 0)
            {
                Bitmap b2;
                if ((otocit % 2) == 0)
                    b2 = new Bitmap(b.Width, b.Height);
                else
                    b2 = new Bitmap(b.Height, b.Width);

                Graphics g = Graphics.FromImage(b2);
                if (otocit == 1)
                    g.TranslateTransform(0, b.Width - 1);
                else if (otocit == 2)
                    g.TranslateTransform(b.Width - 1, b.Height - 1);
                else if (otocit == 3)
                    g.TranslateTransform(b.Height - 1, 0);

                g.RotateTransform(-90 * otocit);
                g.DrawImage(b, 0, 0, b.Width, b.Height);
                b = b2;
            }
            if (grayscale == true)
            {
                Bitmap b2 = new Bitmap(b.Width, b.Height);
                Graphics g = Graphics.FromImage(b2);
                g.DrawImage(b, 0, 0, b.Width, b.Height);
                g.FillRectangle(new SolidBrush(Color.FromArgb(128, 0, 0, 0)), 0, 0, b.Width, b.Height);
                b = b2;
            }
            
            // Řešení lupy (zoomu)
            if (zobrazitVse == true)
            {
                zoom = (double)panel1.ClientSize.Width / (double)b.Width;
                double zoom2 = (double)panel1.ClientSize.Height / (double)b.Height;
                zoom = Math.Min(zoom, zoom2);
            }
            zoomToolStripMenuItem.Text = "Zoom: " + Math.Round(zoom, 2).ToString() + "x";

            // Vykreslení obrázku
            pictureBox1.Width = (int)((double)b.Width * zoom);
            pictureBox1.Height = (int)((double)b.Height * zoom);
            pictureBox1.BackgroundImageLayout = ImageLayout.Stretch;
            pictureBox1.BackgroundImage = b;

            // Načte další, předchozí atd. bitmapy
            if (nacteno == true)
                CacheBitmaps();
        }

        /// <summary>
        /// Metoda pro průběžné načítání dalších obrázků (první a poslední se načtou jen jednou)
        /// </summary>
        private void CacheBitmaps()
        {
            Application.DoEvents();
            try
            {
                tempNextBitmap = null;
                tempPrevBitmap = null;

                if (poprvePosledni == true)
                {
                    tempFirstBitmap = new Bitmap(Image.FromFile(soubory[0].ToString()));
                    Application.DoEvents();
                    tempLastBitmap = new Bitmap(Image.FromFile(soubory[soubory.Length - 1].ToString()));
                    Application.DoEvents();
                    poprvePosledni = false;
                }
                if (kolikaty < (soubory.Length - 1))
                {
                    tempNextBitmap = new Bitmap(Image.FromFile(soubory[kolikaty + 1].ToString()));
                    Application.DoEvents();
                    tempNextIndex = kolikaty + 1;
                }
                if (kolikaty > 0)
                {
                    tempPrevBitmap = new Bitmap(Image.FromFile(soubory[kolikaty - 1].ToString()));
                    Application.DoEvents();
                    tempPrevIndex = kolikaty - 1;
                }
            }
            catch { }
        }

        private void Form2_Resize(object sender, EventArgs e)
        {
            Vykreslit();
        }

        private void vlastnostiObrázkuToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //Zobrazení vlastností obrázku
            StringBuilder sb = new StringBuilder();
            sb.Append(cesta);
            sb.Append("\n");
            sb.Append(cesta.Substring(cesta.LastIndexOf('\\') + 1));
            sb.Append("\n");
            sb.Append(File.GetCreationTime(cesta).ToString());
            sb.Append("\n");
            sb.Append(pictureBox1.BackgroundImage.Width.ToString() + " x " + pictureBox1.BackgroundImage.Height.ToString());
            sb.Append("\n");
            FileInfo fi = new FileInfo(cesta);
            sb.Append(fi.Length.ToString() + " byte");
            sb.Append("\n");
            sb.Append("Zoom: " + Math.Round(zoom, 2).ToString() + "x");
            sb.Append("\n");
            sb.Append((kolikaty + 1) + ". v adresáři, " + kolikCelkem + " celkem.");

            MessageBox.Show(sb.ToString());
        }

        #region Volání metod navigace
        private void dalšíToolStripMenuItem_Click(object sender, EventArgs e)
        {
            NavDalsi();    
        }

        private void předchozíToolStripMenuItem_Click(object sender, EventArgs e)
        {
            NavPredchozi();  
        }
             
        private void posledníToolStripMenuItem_Click(object sender, EventArgs e)
        {
            NavPosledni();
        }
        
        private void prvníToolStripMenuItem_Click(object sender, EventArgs e)
        {
            NavPrvni();
        }
        
        private void přiblížitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            NavPriblizit();
        }
        
        private void zobrazitVšeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            NavZobrazitVse();
        }
        
        private void oddálitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            NavOddalit();
        }
        #endregion

        private void pictureBox1_DoubleClick(object sender, EventArgs e)
        {
            // Uzavře okno
            this.Close();
        }

        private void xUkončitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // Ukončí celý program
            parentForm.Close();
        }

        private void Form2_KeyDown(object sender, KeyEventArgs e)
        {
            // Ovládání klávesnicí
            if (e.KeyCode == Keys.Home)
                NavPrvni();
            else if (e.KeyCode == Keys.End)
                NavPosledni();
            else if (e.KeyCode == Keys.PageUp)
                NavPredchozi();
            else if (e.KeyCode == Keys.PageDown)
                NavDalsi(); 
            else if (e.KeyValue == 107) //+
                NavPriblizit();
            else if (e.KeyValue == 109) //-
                NavOddalit();
            else if (e.KeyValue == 106) //*, 1:1
            {
                zobrazitVse = false;
                zoom = 1d;
                Vykreslit();
            }
            else if (e.KeyValue == 111) // děleno, zobrazit vše
                NavZobrazitVse();
            else if (e.KeyValue == 13) // enter
                this.Close();
            else if (e.KeyValue == 27) // esc
            {
                if (timer1.Enabled == true)
                {
                    timer1.Enabled = false;
                    this.WindowState = FormWindowState.Normal;
                }
                else
                    parentForm.Close();
            }
            else if (e.KeyValue == 72) // H (překlopí horizontálně)
            {
                if (preklopitHor == true)                
                    preklopitHor = false;
                else
                    preklopitHor = true;
                Vykreslit();
            }
            else if (e.KeyValue == 86) // V (překlopí vertikálně)
            {
                if (preklopitVer == true)
                    preklopitVer = false;
                else
                    preklopitVer = true;
                Vykreslit();
            }
            else if (e.KeyValue == 76) // L (otočí doleva)
            {
                otocit++;
                if (otocit > 3)
                    otocit = 0;
                Vykreslit();
            }
            else if (e.KeyValue == 82) // R (otočí doleva)
            {
                otocit--;
                if (otocit < 0)
                    otocit = 3;
                Vykreslit();
            }
            else if (e.KeyValue == 83) // S (slideshow)
            {
                if (timer1.Enabled == false)
                {
                    timer1.Enabled = true;
                    this.WindowState = FormWindowState.Maximized;
                }
                else
                {
                    timer1.Enabled = false;
                    this.WindowState = FormWindowState.Normal;
                }
            }
            else if (e.KeyValue == 71) // G (grayscale)
            {
                if (grayscale == false)
                    grayscale = true;
                else
                    grayscale = false;
                Vykreslit();
            }
        }

        /// <summary>
        /// Metody pro navigaci mezi fotkami
        /// </summary>
        private void NavDalsi()
        {
            if (kolikaty < (soubory.Length - 1))
                Presunout(kolikaty + 1);
        }

        private void NavPredchozi()
        {
            if (kolikaty > 0)
                Presunout(kolikaty - 1);
        }

        private void Presunout(int p)
        {
            kolikaty = p;
            if (soubory[kolikaty] != "")
            {
                this.Text = soubory[kolikaty] + " ... " + (kolikaty + 1) + ". v adresáři, " + kolikCelkem + " celkem.";
                cesta = soubory[kolikaty];
            }
            Vykreslit();
        }

        private void NavPosledni()
        {
            Presunout(soubory.Length - 1);
        }

        private void NavPrvni()
        {
            Presunout(0);
        }

        private void NavPriblizit()
        {
            zobrazitVse = false;
            zoom *= (5d / 4d);
            Vykreslit();
        }

        private void NavZobrazitVse()
        {
            zobrazitVse = true;
            Vykreslit();
        }

        private void NavOddalit()
        {
            zobrazitVse = false;
            zoom *= (4d / 5d);
            Vykreslit();
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (kolikaty < (soubory.Length - 1))
                NavDalsi();
            else
                NavPrvni();
        }

        private void nápovědaToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // Nápověda k oknu zobrazení
            MessageBox.Show("Toto je okno zobrazení fotografií.\n\nMůžete klikat na nabídku nahoře - zvolit přechod na další fotku, předchozí atd. Můžete přiblížit, oddálit obrázek, zobrazit celý. Program lze ovládat i klávesnicí, PageUp a PageDown přecházíte mezi snímky, Home a End = první a poslední, + a - přiblíží nebo oddálí obrázek, * zobrazí 1:1, / zobrazí optimálně do okna, Enter přejdete do procházení adresářů, Esc vypnete program. Klávesou H a V můžete obrázek překlápět, L a R obrázek otáčí. Příští obrázky a první a poslední se průběžně načítají, aby se pak zobrazily rychleji. Klávesou S spustíte slideshow - prohlížení fotek po 3 sekundách, slideshow ukončíte klávesou Esc nebo znovu S. Klávesou G zobrazíte černobíle (to ještě zlobí).");
        }
    }
}
